/*
 Expandable KVM
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 8/07/2014
 Last Modified: 30/10/2014
 
 Uses an ATtiny24 to monitor the buttons and once they are pressed, we send a signal on the HighZ read/enable pin 
 to tell the other KVMs to go to HighZ mode, then switch in the analog switch to the PC selected and light the LED.
 
 */

// ATtiny24/44/84 Pin map
//
//                                 +-\/-+
//                           VCC  1|o   |14  GND
//                           PB0  2|    |13  PA0 Button 1
//                           PB1  3|    |12  PA1 Button 2
//                     Reset PB3  4|    |11  PA2 LED 1
//         HighZ Read/Enable PB2  5|    |10  PA3 LED 2
//    Analog switch 2 select PA7  6|    |9   PA4 Quad analog switch enable
//    Analog switch 1 select PA6  7|    |8   PA5 Quad analog switch input
//                                 +----+

#define F_CPU 1000000 // 1 MHz

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include "setup.c"

int main(void) {
	setup();
	
	// On power up blink the LED 3 times
	for (int x = 0; x < 3; x++) {
		PORTA |= (1<<LED1) | (1<<LED2);
		_delay_ms(100);
		PORTA &= ~((1<<LED1) | (1<<LED2));
		_delay_ms(100);
	}
	
	while(1) {
		system_sleep();
		_delay_ms(50); // Wait for button debounce
		
		// Check if HighZ pin woke us up
		if (PINB & (1<<HIGH_Z)) {
			// High-Z all analog switches
			PORTA |= (1<<QUAD_SW_ENABLE);
			PORTA |= (1<<ANA_SW1_SELECT) | (1<<ANA_SW2_SELECT);
			PORTA &= ~(1<<LED1);
			PORTA &= ~(1<<LED2);
		}
		else { // Check for button pushes
			if (PINA & (1<<SW1)) { // Change to PC1
				PORTA |= (1<<LED1);
				PORTA &= ~(1<<LED2);
				
				PORTA &= ~(1<<QUAD_SW_ENABLE);
				PORTA &= ~(1<<QUAD_SW_IN); // Low - PC1
				PORTA |= (1<<ANA_SW2_SELECT); // Disable switch 2
				PORTA &= ~(1<<ANA_SW1_SELECT);
				
				// Set HighZ pin high for 100ms
				cli(); // Turn interrupts off
				DDRB |= (1<<HIGH_Z);
				PORTB |= (1<<HIGH_Z);
				_delay_ms(100);
				PORTB &= ~(1<<HIGH_Z);
				DDRB &= ~(1<<HIGH_Z);
				sei(); // Turn interrupts back on
			}
			else if (PINA & (1<<SW2)) { // Change to PC2
				PORTA |= (1<<LED2);
				PORTA &= ~(1<<LED1);
				
				PORTA &= ~(1<<QUAD_SW_ENABLE);
				PORTA |= (1<<QUAD_SW_IN); // High - PC2
				PORTA |= (1<<ANA_SW1_SELECT); // Disable switch 1
				PORTA &= ~(1<<ANA_SW2_SELECT);
				
				// Set HighZ pin high for 100ms
				cli(); // Turn interrupts off
				DDRB |= (1<<HIGH_Z);
				PORTB |= (1<<HIGH_Z);
				_delay_ms(100);
				PORTB &= ~(1<<HIGH_Z);
				DDRB &= ~(1<<HIGH_Z);
				sei(); // Turn interrupts back on
			}
		}
	}
}

// Pin interrupt on switches
ISR(PCINT0_vect) { }

// Pin interrupt on HighZ reading pin
ISR(PCINT1_vect) { }
